@@ -19,6 +19,7 @@
 #undef HAVE_IPV6
 #undef HAVE_POPT_H
 #undef HAVE_SOCKS_H
+#undef HAVE_PERL
 
 /* macros/curses checks */
 #undef HAS_CURSES
@@ -33,6 +34,7 @@
 /* our own curses checks */
 #undef HAVE_NCURSES_USE_DEFAULT_COLORS
 #undef HAVE_CURSES_IDCOK
+#undef HAVE_CURSES_RESIZETERM
 
 /* nls */
 #undef ENABLE_NLS
