@@ -59,7 +59,7 @@ static void save_channels(void)
 	if (rec->background != NULL) config_set_str(chprop, "background", rec->background);
 	if (rec->font != NULL) config_set_str(chprop, "font", rec->font);
     }
-    PLSave(cprop, TRUE);
+    if (!readonly) PLSave(cprop, TRUE);
 }
 
 static void sig_join(GtkWidget *clist)
@@ -191,7 +191,19 @@ static GtkWidget *channel_properties_dialog(GtkWidget *clist, SETUP_CHANNEL_REC
                     (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
 
   if (ircnets != NULL)
-    gtk_combo_set_popdown_strings(GTK_COMBO(cboIRCNet), ircnets);
+  {
+      GList *tmp, *list;
+
+      list = NULL;
+      for (tmp = ircnets; tmp != NULL; tmp = tmp->next)
+      {
+          IRCNET_REC *ircnet = tmp->data;
+
+          list = g_list_append(list, ircnet->name);
+      }
+      gtk_combo_set_popdown_strings(GTK_COMBO(cboIRCNet), list);
+      g_list_free(list);
+  }
 
   ircnet_entry = GTK_WIDGET(GTK_COMBO(cboIRCNet)->entry);
   gtk_object_set_data(GTK_OBJECT(dialog), "ircnet", ircnet_entry);
