@@ -132,7 +132,7 @@ static void save_log_setup(void)
 	    config_set_int(subiprop, "level", item->level);
         }
     }
-    PLSave(cprop, TRUE);
+    if (!readonly) PLSave(cprop, TRUE);
 }
 
 static void load_log_setup(void)
@@ -638,11 +638,11 @@ void dialog_log(void)
     button = gtk_button_new_with_label(_("Clear all"));
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(sig_clear_all_levels), ctree);
-    gtk_table_attach_defaults(GTK_TABLE(level_table), button, 0, 1, 11, 12);
+    gtk_table_attach_defaults(GTK_TABLE(level_table), button, 0, 1, 12, 13);
     button = gtk_button_new_with_label(_("Select all"));
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(sig_select_all_levels), ctree);
-    gtk_table_attach_defaults(GTK_TABLE(level_table), button, 1, 2, 11, 12);
+    gtk_table_attach_defaults(GTK_TABLE(level_table), button, 1, 2, 12, 13);
 
     /* auto-open log button */
     autolog_button = gtk_check_button_new_with_label(_("Open log at startup"));
