@@ -245,7 +245,7 @@ static gint sig_keypress(GtkWidget *widget, GdkEventKey *event, MAIN_WINDOW_REC
     ret = key_pressed(key, window);
     g_free(key);
 
-    if (ret && (event->keyval < 32 || event->keyval > 255) && event->keyval != GDK_Return && event->keyval != GDK_KP_Enter)
+    if (ret && ((event->state & (GDK_CONTROL_MASK|GDK_MOD1_MASK)) != 0 || event->keyval < 32 || event->keyval > 255) && event->keyval != GDK_Return && event->keyval != GDK_KP_Enter)
     {
 	/* key was used, stop signal */
 	gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
@@ -330,6 +330,8 @@ static void sig_entry_activated(GtkWidget *entry, MAIN_WINDOW_REC *window)
             cur_channel = window->active->active;
             signal_emit("channel changed", 1, cur_channel);
         }
+
+	ui_translate_output(str);
         signal_emit("send command", 3, str, cur_channel->server, cur_channel);
 
         str = ptr;
@@ -391,21 +393,17 @@ static void sig_entry_deleted(GtkWidget *entry, gint start_pos, gint end_pos, MA
 
 static void term_change_pos(MAIN_WINDOW_REC *window)
 {
-    /* window moved - redraw the transparent windows */
+    /* window moved - redraw the transparent window */
+    WINDOW_REC *rec;
     GtkWidget *itext;
-    GList *tmp;
 
-    for (tmp = window->children; tmp != NULL; tmp = tmp->next)
+    rec = window->active;
+    if (WINDOW_VIEW(rec)->itext)
     {
-        WINDOW_REC *rec = tmp->data;
+	itext = WINDOW_VIEW(rec)->text;
 
-        if (WINDOW_VIEW(rec)->itext)
-        {
-            itext = WINDOW_VIEW(rec)->text;
-
-            if (itext->window != NULL && (GTK_ITEXT(itext)->bg_flags & ITEXT_BG_TRANSPARENT))
-                gtk_widget_queue_draw(itext);
-        }
+	if (itext->window != NULL && (GTK_ITEXT(itext)->bg_flags & ITEXT_BG_TRANSPARENT))
+	    gtk_widget_queue_draw(itext);
     }
 }
 
@@ -638,6 +636,12 @@ static gboolean sig_next_window(void)
     return TRUE;
 }
 
+static gboolean sig_window_goto_active(void)
+{
+    signal_emit("command window goto", 3, "active", cur_channel->server, cur_channel);
+    return TRUE;
+}
+
 static gboolean sig_prev_channel(void)
 {
     signal_emit("command channel prev", 3, "", cur_channel->server, cur_channel);
@@ -783,6 +787,7 @@ void gui_mainwindows_init(void)
     key_bind("check replaces", NULL, NULL, "Return", (SIGNAL_FUNC) sig_replace);
     key_bind("window prev", NULL, _("Previous window"), "CTRL-P", (SIGNAL_FUNC) sig_prev_window);
     key_bind("window next", NULL, _("Next window"), "CTRL-N", (SIGNAL_FUNC) sig_next_window);
+    key_bind("window active", NULL, "Go to next window with the highest activity", "ALT-A", (SIGNAL_FUNC) sig_window_goto_active);
     key_bind("channel next", NULL, "Next channel", "CTRL-X", (SIGNAL_FUNC) sig_next_channel);
     key_bind("channel prev", NULL, "Next channel", NULL, (SIGNAL_FUNC) sig_prev_channel);
 
@@ -813,6 +818,7 @@ void gui_mainwindows_deinit(void)
     key_unbind("check replaces", (SIGNAL_FUNC) sig_replace);
     key_unbind("window prev", (SIGNAL_FUNC) sig_prev_window);
     key_unbind("window next", (SIGNAL_FUNC) sig_next_window);
+    key_unbind("window active", (SIGNAL_FUNC) sig_window_goto_active);
     key_unbind("channel next", (SIGNAL_FUNC) sig_next_channel);
     key_unbind("channel prev", (SIGNAL_FUNC) sig_prev_channel);
 
