@@ -508,7 +508,7 @@ static void menu_dcc_send(GtkWidget *menuitem, MAIN_WINDOW_REC *window)
 static void menu_window_new(GtkWidget *menuitem, MAIN_WINDOW_REC *window)
 {
     channel_create(window == NULL ? NULL : window->active->active->server,
-		   _("(empty)"), CHANNEL_TYPE_EMPTY);
+		   _("(empty)"), CHANNEL_TYPE_EMPTY, FALSE);
 }
 
 static void menu_window_new_window(GtkWidget *menuitem, MAIN_WINDOW_REC *window)
@@ -581,12 +581,14 @@ static void menu_settings_themes(void)
     setup("setup themes", &dialog, NULL);
 }
 
+#if 0
 static void menu_settings_session(void)
 {
     static GtkWidget *dialog = NULL;
 
     setup("setup session", &dialog, NULL);
 }
+#endif
 
 static void menu_settings_keyboard(void)
 {
