@@ -295,6 +295,7 @@ static gint sig_topicentry_press(GtkWidget *entry, GdkEventButton *event)
 static void sig_topicentry_activated(GtkWidget *entry, GUI_WINDOW_REC *gui)
 {
     CHANNEL_REC *channel;
+    gchar *str;
 
     g_return_if_fail(entry != NULL);
     g_return_if_fail(gui != NULL);
@@ -309,8 +310,10 @@ static void sig_topicentry_activated(GtkWidget *entry, GUI_WINDOW_REC *gui)
         return;
     }
 
-    signal_emit("command topic", 3, gtk_entry_get_text(GTK_ENTRY(entry)),
-		channel->server, channel);
+    str = g_strdup_printf("TOPIC %s :%s", channel->name,
+			  gtk_entry_get_text(GTK_ENTRY(entry)));
+    irc_send_cmd(cur_channel->server, str);
+    g_free(str);
 }
 
 static void sig_modebutton_toggled(GtkToggleButton *button, gint mode)
@@ -505,8 +508,8 @@ static void sig_tab_left(GtkWidget *widget, GUI_WINDOW_REC *gui)
     gtk_notebook_reorder_child(notebook, gui->window, num-1);
 
     /* redraw the window tabs so that window numbers gets redrawed */
-    signal_emit("channel change name", 2, gui->active->window->active, gui->active->window->active->name);
-    signal_emit("channel change name", 2, prevwin->active->window->active, prevwin->active->window->active->name);
+    signal_emit("channel name changed", 2, gui->active->window->active, gui->active->window->active->name);
+    signal_emit("channel name changed", 2, prevwin->active->window->active, prevwin->active->window->active->name);
 }
 
 static void sig_tab_right(GtkWidget *widget, GUI_WINDOW_REC *gui)
@@ -537,8 +540,8 @@ static void sig_tab_right(GtkWidget *widget, GUI_WINDOW_REC *gui)
     gtk_notebook_reorder_child(notebook, gui->window, num+1);
 
     /* redraw the window tabs so that window numbers gets redrawed */
-    signal_emit("channel change name", 2, gui->active->window->active, gui->active->window->active->name);
-    signal_emit("channel change name", 2, nextwin->active->window->active, nextwin->active->window->active->name);
+    signal_emit("channel name changed", 2, gui->active->window->active, gui->active->window->active->name);
+    signal_emit("channel name changed", 2, nextwin->active->window->active, nextwin->active->window->active->name);
 }
 
 static void gui_window_new(WINDOW_REC *window)
@@ -735,7 +738,7 @@ static void gui_window_new(WINDOW_REC *window)
     signal_emit("gui window created", 1, window);
 }
 
-static gboolean gui_window_created(WINDOW_REC *window)
+static gboolean gui_window_created(WINDOW_REC *window, gpointer automatic)
 {
     MAIN_WINDOW_REC *parent;
     WINDOW_VIEW_REC *view;
@@ -745,7 +748,8 @@ static gboolean gui_window_created(WINDOW_REC *window)
     g_return_val_if_fail(window != NULL, FALSE);
 
     parent = (!setup_get_bool("toggle_use_tabbed_windows") || cur_channel == NULL || window_create_override == 0) &&
-	window_create_override < 1 ? gui_mainwindow_create(window) : WINDOW_GUI(CHANNEL_PARENT(cur_channel))->parent;
+	window_create_override < 1 ? gui_mainwindow_create(window) :
+	(automatic ? mainwindows->data : WINDOW_GUI(CHANNEL_PARENT(cur_channel))->parent);
 
     view = g_new0(WINDOW_VIEW_REC, 1);
     window->gui_data = view;
