@@ -1193,24 +1193,28 @@ static void draw(GtkWidget *widget, gint xp1, gint yp1, gint xp2, gint yp2)
 
     itext = GTK_ITEXT(widget);
 
-    if (!GTK_WIDGET_MAPPED(widget) || !GTK_WIDGET_DRAWABLE(widget))
-	return;
-
     /* first add the missing lines .. */
     if (itext->width >= itext->min_width && itext->lines_add != NULL)
 	gtk_itext_append_lines(itext);
 
+    y = yp1 < itext->font_height ? 0 : yp1/itext->font_height-1;
+    pos = gtk_itext_get_line(itext, itext->starty+y);
+
+    if (!GTK_WIDGET_MAPPED(widget) || !GTK_WIDGET_DRAWABLE(widget))
+    {
+	/* window isn't visible, don't bother trying to draw it.. */
+	return;
+    }
+
     /* clear area */
     gtk_itext_set_origin(itext);
-    gdk_draw_rectangle(GTK_WIDGET(itext)->window, itext->back_gc, 1,
+    gdk_draw_rectangle(widget->window, itext->back_gc, 1,
 		       xp1, yp1, xp2-xp1+1, yp2-yp1+1);
 
-    y = yp1 < itext->font_height ? 0 : yp1/itext->font_height-1;
     y2 = yp2/itext->font_height;
     if (yp2 >= itext->height) y2--;
     d(printf("draw %d..%d\n", y, y2));
 
-    pos = gtk_itext_get_line(itext, itext->starty+y);
     for (; pos != NULL && y <= y2; pos = pos->next, y++)
     {
 	line = pos->data;
@@ -1348,8 +1352,8 @@ static void gtk_itext_draw(GtkWidget *widget, GdkRectangle *area)
 
     if (itext->bg_flags & ITEXT_BG_TRANSPARENT)
     {
-	itext_transparent_free(itext);
-	itext->pixmap = get_background(itext);
+	/*itext_transparent_free(itext);
+	itext->pixmap = get_background(itext);*/
 
 	XTranslateCoordinates (GDK_WINDOW_XDISPLAY(widget->window),
 			       GDK_WINDOW_XWINDOW(widget->window),
@@ -2214,12 +2218,12 @@ static void gtk_itext_init_background(GtkIText *itext)
     if (!GTK_WIDGET_REALIZED(itext))
 	return;
 
-    if (itext->pixmap != NULL)
+    if (itext->pixmap != NULL && (itext->bg_flags & ITEXT_BG_TRANSPARENT) == 0)
     {
 	/* destroy old pixmap */
-	if (itext->bg_flags & ITEXT_BG_TRANSPARENT)
+	/*if (itext->bg_flags & ITEXT_BG_TRANSPARENT)
 	    itext_transparent_free(itext);
-	else
+	else*/
 	    gdk_pixmap_unref(itext->pixmap);
 	itext->pixmap = NULL;
     }
@@ -2228,7 +2232,7 @@ static void gtk_itext_init_background(GtkIText *itext)
     if (itext->bg_flags & ITEXT_BG_TRANSPARENT)
     {
 #ifdef TRANSPARENCY
-	itext->pixmap = get_background(itext);
+	if (itext->pixmap == NULL) itext->pixmap = get_background(itext);
 #endif
     }
     else if (itext->pixmap_path != NULL)
