@@ -275,7 +275,7 @@ static gboolean sig_setup_apply_again(GtkWidget *dialog)
 		g_string_sprintf(str, "extra_blue_%d", num-BASIC_COLORS);
 	    config_set_int(colprop, str->str, THEME_GUI(rec)->colors[num].blue);
 	}
-	PLSave(tprop, TRUE);
+	if (!readonly) PLSave(tprop, TRUE);
 	PLRelease(tprop);
     }
     g_string_free(str, TRUE);
@@ -444,8 +444,7 @@ static gboolean setup_theme_redraw(THEME_REC *theme, gpointer init)
     }
     while (n != BASIC_COLORS);
     gtk_clist_thaw(clist);
-    set_color(gtk_object_get_data(GTK_OBJECT(setup_dialog), "eventbox"), 100, theme);
-
+    gtk_clist_select_row(GTK_CLIST(clist), 0, 0);
     return TRUE;
 }
 
