@@ -61,7 +61,7 @@ static SETUP_SERVER_REC *add_server_to_list(GList **servers, gchar *ircnet, gcha
   
   if(replacerec != NULL) {
     if((tmp = g_list_find(*servers, replacerec)) != NULL) {
-      rec = (SETUP_SERVER_REC *)tmp->data;
+      rec = tmp->data;
       g_free(rec->ircnet);
       g_free(rec->server);
       g_free(rec->password);
@@ -75,6 +75,15 @@ static SETUP_SERVER_REC *add_server_to_list(GList **servers, gchar *ircnet, gcha
     } else
       rec = NULL;
   } else {
+    for (tmp = *servers; tmp != NULL; tmp = tmp->next)
+    {
+	rec = tmp->data;
+
+	if (g_strcasecmp(rec->server, server) == 0 &&
+	    g_strcasecmp(rec->ircnet, ircnet) == 0 && rec->port == port)
+	    return NULL;
+    }
+
     rec = g_new(SETUP_SERVER_REC, 1);
     rec->ircnet = g_strdup(ircnet);
     rec->server = g_strdup(server);
@@ -136,7 +145,7 @@ static gboolean sig_setup_apply(GtkWidget *dialog)
     config_set_int(iprop, "cmd_queue_speed", rec->cmd_queue_speed);
   }
 
-  PLSave(cprop, TRUE);
+  if (!readonly) PLSave(cprop, TRUE);
 
   return TRUE;
 }
@@ -306,8 +315,20 @@ static void server_dialog(SETUP_SERVER_REC *server, GtkSignalFunc func)
 
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), table2, TRUE, TRUE, 0);
 
-  if (ircnets != NULL)
-    gtk_combo_set_popdown_strings(GTK_COMBO(cboIRCNet), temp_ircnets);
+  if (temp_ircnets != NULL)
+  {
+      GList *tmp, *list;
+
+      list = NULL;
+      for (tmp = temp_ircnets; tmp != NULL; tmp = tmp->next)
+      {
+          IRCNET_REC *ircnet = tmp->data;
+
+          list = g_list_append(list, ircnet->name);
+      }
+      gtk_combo_set_popdown_strings(GTK_COMBO(cboIRCNet), list);
+      g_list_free(list);
+  }
 
   if(server != NULL) {
     gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(cboIRCNet)->entry), server->ircnet);
