@@ -3,10 +3,10 @@
 
 #define CHANNEL_GUI(a) ((GUI_CHANNEL_REC *) ((a)->gui_data))
 
-typedef struct
+/*typedef struct
 {
 }
-GUI_CHANNEL_REC;
+GUI_CHANNEL_REC;*/
 
 void gui_channels_init(void);
 void gui_channels_deinit(void);
