@@ -3,8 +3,12 @@
 
 typedef void (*STATUSBAR_FUNC) (gint xpos, gint ypos, gint size);
 
+/* create new statusbar, return position */
+gint gui_statusbar_create(gboolean up);
+void gui_statusbar_delete(gboolean up, gint ypos);
+
 /* allocate area in statusbar, returns tag or -1 if failed */
-gint gui_statusbar_allocate(gint size, gboolean right_justify, STATUSBAR_FUNC func);
+gint gui_statusbar_allocate(gint size, gboolean right_justify, gboolean up, gint ypos, STATUSBAR_FUNC func);
 void gui_statusbar_resize(gint tag, gint size);
 void gui_statusbar_remove(gint tag);
 
