@@ -9,6 +9,9 @@
 
 #define ATTR_UNDERLINE 0x100
 #define ATTR_COLOR8    0x200
+#define ATTR_REVERSE   0x400
+
+extern gboolean use_colors;
 
 gint init_screen(void); /* Initialize screen, detect screen length */
 void deinit_screen(void); /* Deinitialize screen */
