@@ -28,16 +28,18 @@ void command_bind(gchar *cmd, gchar *category, SIGNAL_FUNC func)
     COMMAND_REC *rec;
 
     g_return_if_fail(cmd != NULL);
-    g_return_if_fail(func != NULL);
 
     rec = g_new0(COMMAND_REC, 1);
     rec->cmd = g_strdup(cmd);
     rec->category = category == NULL ? NULL : g_strdup(category);
     commands = g_list_append(commands, rec);
 
-    cmd = g_strconcat("command ", cmd, NULL);
-    signal_add(cmd, func);
-    g_free(cmd);
+    if (func != NULL)
+    {
+	cmd = g_strconcat("command ", cmd, NULL);
+	signal_add(cmd, func);
+	g_free(cmd);
+    }
 
     signal_emit("commandlist new", 1, rec);
 }
@@ -57,7 +59,6 @@ void command_unbind(gchar *cmd, SIGNAL_FUNC func)
     GList *tmp;
 
     g_return_if_fail(cmd != NULL);
-    g_return_if_fail(func != NULL);
 
     for (tmp = commands; tmp != NULL; tmp = tmp->next)
     {
@@ -70,14 +71,17 @@ void command_unbind(gchar *cmd, SIGNAL_FUNC func)
 	}
     }
 
-    cmd = g_strconcat("command ", cmd, NULL);
-    signal_remove(cmd, func);
-    g_free(cmd);
+    if (func != NULL)
+    {
+	cmd = g_strconcat("command ", cmd, NULL);
+	signal_remove(cmd, func);
+	g_free(cmd);
+    }
 }
 
 void command_runsub(gchar *cmd, gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
 {
-    gchar *subcmd, *args;
+    gchar *subcmd, *defcmd, *args;
 
     g_return_if_fail(data != NULL);
 
@@ -90,9 +94,10 @@ void command_runsub(gchar *cmd, gchar *data, SERVER_REC *server, CHANNEL_REC *ch
     g_strdown(subcmd);
     if (!signal_emit(subcmd, 3, args, server, channel))
     {
-	g_free(subcmd);
-	subcmd = g_strdup_printf("default command %s", cmd);
-	signal_emit(subcmd, 3, data, server, channel);
+	defcmd = g_strdup_printf("default command %s", cmd);
+	if (!signal_emit(subcmd, 3, data, server, channel))
+	    signal_emit("unknown command", 3, strchr(subcmd, ' ')+1, server, channel);
+	g_free(defcmd);
     }
     g_free(subcmd);
 }
@@ -157,7 +162,7 @@ void command_split(SERVER_REC *server, gchar *cmd, gint arg, gint max_nicks)
     g_free(pre);
 }
 
-static gboolean cmd_connect(gchar *data)
+static gboolean connect_server(gchar *data, gchar *channels)
 {
     SERVER_REC *server;
     gchar *params, *addr, *portstr, *password, *nick;
@@ -175,7 +180,7 @@ static gboolean cmd_connect(gchar *data)
 	sscanf(portstr, "%d", &port);
 
     /* connect to server */
-    server = server_connect(addr, port, password, nick);
+    server = server_connect(addr, port, password, nick, channels);
 
     if (cur_channel->server == NULL || cur_channel->type == CHANNEL_TYPE_EMPTY)
     {
@@ -187,12 +192,20 @@ static gboolean cmd_connect(gchar *data)
     return TRUE;
 }
 
+static gboolean cmd_connect(gchar *data)
+{
+    return connect_server(data, NULL);
+}
+
 static gboolean cmd_disconnect(gchar *data, SERVER_REC *server)
 {
     gchar *params, *tag, *msg;
 
     g_return_val_if_fail(data != NULL, FALSE);
 
+    if (g_strncasecmp(data, "RECON-", 6) == 0)
+	return TRUE; /* remove reconnection, handle in server-reconnect.c */
+
     params = cmd_get_params(data, 2 | PARAM_FLAG_GETREST, &tag, &msg);
     if (*tag != '\0' && strcmp(tag, "*") != 0) server = server_find_tag(tag);
     if (server == NULL) cmd_param_error(CMDERR_NOT_CONNECTED);
@@ -217,13 +230,21 @@ static gboolean cmd_disconnect(gchar *data, SERVER_REC *server)
 
 static gboolean cmd_server(gchar *data, SERVER_REC *server)
 {
+    gchar *channels;
+
     g_return_val_if_fail(data != NULL, FALSE);
     if (*data == '\0') cmd_return_error(CMDERR_NOT_ENOUGH_PARAMS);
 
-    if (*data != '+' && server != NULL)
-        cmd_disconnect("* Changing server", server);
+    if (*data == '+' || server == NULL)
+	channels = NULL;
+    else
+    {
+	channels = server_get_channels(server);
+	cmd_disconnect("* Changing server", server);
+    }
 
-    cmd_connect(data + (*data == '+' ? 1 : 0));
+    connect_server(data + (*data == '+' ? 1 : 0), channels);
+    if (channels != NULL) g_free(channels);
 
     return TRUE;
 }
@@ -280,7 +301,7 @@ static gboolean cmd_ctcp(gchar *data, SERVER_REC *server)
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
 
     params = cmd_get_params(data, 3 | PARAM_FLAG_GETREST, &target, &ctcpcmd, &ctcpdata);
-    if (*target == '\0' || *ctcpcmd == '\0') cmd_return_error(CMDERR_NOT_ENOUGH_PARAMS);
+    if (*target == '\0' || *ctcpcmd == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
     g_strup(ctcpcmd);
     if (*ctcpdata == '\0')
@@ -301,7 +322,7 @@ static gboolean cmd_nctcp(gchar *data, SERVER_REC *server)
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
 
     params = cmd_get_params(data, 3 | PARAM_FLAG_GETREST, &target, &ctcpcmd, &ctcpdata);
-    if (*target == '\0' || *ctcpcmd == '\0') cmd_return_error(CMDERR_NOT_ENOUGH_PARAMS);
+    if (*target == '\0' || *ctcpcmd == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
     g_strup(ctcpcmd);
     g_string_sprintf(tmpstr, "NOTICE %s :\001%s %s\001", target, ctcpcmd, ctcpdata);
@@ -332,7 +353,7 @@ static gboolean cmd_quit(gchar *data)
     return TRUE;
 }
 
-static gboolean cmd_join(gchar *data, SERVER_REC *server)
+static gboolean cmd_join(gchar *data, SERVER_REC *server, CHANNEL_REC *cur_channel, gboolean automatic)
 {
     CHANNEL_REC *channel;
     gchar *next, *key, *chan;
@@ -370,7 +391,7 @@ static gboolean cmd_join(gchar *data, SERVER_REC *server)
         }
 
 	channel = channel_find(server, chan);
-        if (channel != NULL)
+        if (channel != NULL && channel->type == CHANNEL_TYPE_CHANNEL)
         {
 	    /* already joined this channel.. */
             signal_emit("gui channel open", 1, channel);
@@ -382,7 +403,8 @@ static gboolean cmd_join(gchar *data, SERVER_REC *server)
 	g_string_sprintfa(str, str->len == 5 ? "%s" : ",%s", chan);
         if (key != NULL) g_string_sprintfa(str, " %s", key);
 
-	channel = channel_create(server, chan + ((chan[0] == '!' && chan[1] == '!') ? 1 : 0), CHANNEL_TYPE_CHANNEL);
+	channel = channel_create(server, chan + ((chan[0] == '!' && chan[1] == '!') ? 1 : 0),
+				 CHANNEL_TYPE_CHANNEL, cur_channel == NULL || automatic);
 
 	if (key != NULL)
 	{
@@ -412,7 +434,7 @@ static gboolean cmd_part(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
     g_return_val_if_fail(channel != NULL, FALSE);
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
 
-    params = cmd_get_params(data, 2 | PARAM_FLAG_OPTCHAN, channel, &channame, &msg);
+    params = cmd_get_params(data, 2 | PARAM_FLAG_OPTCHAN | PARAM_FLAG_GETREST, channel, &channame, &msg);
     if (*channame == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
     chanrec = channel_find(server, channame);
@@ -444,29 +466,6 @@ static gboolean cmd_kick(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
     g_string_sprintf(tmpstr, "KICK %s %s :%s", channame, nicks, reason);
     command_split(server, tmpstr->str, 3, server->max_kicks_in_cmd);
 
-#if 0
-    /* split the kicks to max. n in one cmd */
-    nickstr = g_string_new(NULL);
-    nicklist = str2list(nicks, ','); count = 0;
-    for (tmp = nicklist; ; tmp = tmp->next)
-    {
-	if (tmp != NULL)
-	    g_string_sprintfa(nickstr, nickstr->len == 0 ? "%s" : ",%s", (gchar *) tmp->data);
-
-	if (tmp == NULL || ++count == server->max_kicks_in_cmd)
-	{
-	    count = 0;
-	    g_string_sprintf(tmpstr, "KICK %s %s :%s", channame, nickstr->str, reason);
-	    irc_send_cmd(server, tmpstr->str);
-	    if (tmp == NULL) break;
-
-	    g_string_truncate(nickstr, 0);
-	}
-    }
-    g_free(nicklist->data);
-    g_list_free(nicklist);
-    g_string_free(nickstr, TRUE);
-#endif
     g_free(params);
     return TRUE;
 }
@@ -481,12 +480,9 @@ static gboolean cmd_topic(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
 
     params = cmd_get_params(data, 2 | PARAM_FLAG_OPTCHAN | PARAM_FLAG_GETREST, channel, &channame, &topic);
 
-    if (*topic != '\0')
-    {
-        /* Change topic */
-        g_string_sprintf(tmpstr, "TOPIC %s :%s", channame, topic);
-        irc_send_cmd(server, tmpstr->str);
-    }
+    g_string_sprintf(tmpstr, *topic == '\0' ? "TOPIC %s" : "TOPIC %s :%s",
+		     channame, topic);
+    irc_send_cmd(server, tmpstr->str);
 
     g_free(params);
     return TRUE;
@@ -589,7 +585,8 @@ static gboolean cmd_whois(gchar *data, SERVER_REC *server)
     g_return_val_if_fail(data != NULL, FALSE);
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
 
-    if (*data == '\0') cmd_return_error(CMDERR_NOT_ENOUGH_PARAMS);
+    while (*data == ' ') data++;
+    if (*data == '\0') data = server->nick;
 
     g_string_sprintf(tmpstr, "WHOIS %s", data);
     irc_send_cmd(server, tmpstr->str);
@@ -639,6 +636,9 @@ static gboolean cmd_away(gchar *data, SERVER_REC *server)
     g_return_val_if_fail(data != NULL, FALSE);
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
 
+    if (server->away_reason != NULL) g_free(server->away_reason);
+    server->away_reason = g_strdup(data);
+
     g_string_sprintf(tmpstr, "AWAY :%s", data);
     irc_send_cmd(server, tmpstr->str);
 
@@ -732,10 +732,39 @@ static gboolean cmd_wall(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
     return TRUE;
 }
 
+static gboolean cmd_cycle(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
+{
+    CHANNEL_REC *chanrec;
+    gchar *params, *channame, *msg;
+
+    g_return_val_if_fail(data != NULL, FALSE);
+    g_return_val_if_fail(channel != NULL, FALSE);
+    if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
+
+    params = cmd_get_params(data, 2 | PARAM_FLAG_OPTCHAN, channel, &channame, &msg);
+    if (*channame == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
+
+    chanrec = channel_find(server, channame);
+    if (chanrec == NULL || chanrec->type != CHANNEL_TYPE_CHANNEL)
+        cmd_param_error(CMDERR_CHAN_NOT_FOUND);
+
+    g_string_sprintf(tmpstr, "PART %s", channame);
+    if (*msg != '\0') g_string_sprintfa(tmpstr, " :%s", msg);
+    irc_send_cmd(server, tmpstr->str);
+
+    g_string_sprintf(tmpstr, "JOIN %s %s", channame,
+		     chanrec->key == NULL ? "" : chanrec->key);
+    irc_send_cmd(server, tmpstr->str);
+
+    g_free(params);
+    return TRUE;
+}
+
 static gboolean default_cmd(gchar *data, SERVER_REC *server)
 {
     g_return_val_if_fail(data != NULL, FALSE);
-    if (server == NULL) cmd_return_error(CMDERR_NOT_CONNECTED);
+    if (server == NULL || !server->connected)
+        cmd_return_error(CMDERR_NOT_CONNECTED);
 
     irc_send_cmd(server, data);
     return FALSE;
@@ -786,6 +815,7 @@ void commands_init(void)
     command_bind("ison", NULL, (SIGNAL_FUNC) cmd_ison);
     command_bind("quote", NULL, (SIGNAL_FUNC) cmd_quote);
     command_bind("wall", NULL, (SIGNAL_FUNC) cmd_wall);
+    command_bind("cycle", NULL, (SIGNAL_FUNC) cmd_cycle);
     signal_add("default command", (SIGNAL_FUNC) default_cmd);
 }
 
@@ -817,6 +847,7 @@ void commands_deinit(void)
     command_unbind("ison", (SIGNAL_FUNC) cmd_ison);
     command_unbind("quote", (SIGNAL_FUNC) cmd_quote);
     command_unbind("wall", (SIGNAL_FUNC) cmd_wall);
+    command_unbind("cycle", (SIGNAL_FUNC) cmd_cycle);
     signal_remove("default command", (SIGNAL_FUNC) default_cmd);
 
     g_string_free(tmpstr, TRUE);
