@@ -27,7 +27,7 @@ enum
 #define cmd_return_error(a) { signal_emit("error command", 1, GINT_TO_POINTER(a)); return FALSE; }
 #define cmd_param_error(a) { g_free(params); cmd_return_error(a); }
 
-GList *commands;
+extern GList *commands;
 
 void command_bind(gchar *cmd, gchar *category, SIGNAL_FUNC func);
 void command_unbind(gchar *cmd, SIGNAL_FUNC func);
