@@ -31,8 +31,11 @@ gint nicklist_compare(NICK_REC *p1, NICK_REC *p2)
     if (p1->op && !p2->op) return -1;
     if (!p1->op && p2->op) return 1;
 
-    if (p1->voice && !p2->voice) return -1;
-    if (!p1->voice && p2->voice) return 1;
+    if (!p1->op)
+    {
+	if (p1->voice && !p2->voice) return -1;
+	if (!p1->voice && p2->voice) return 1;
+    }
 
     return g_strcasecmp(p1->nick, p2->nick);
 }
@@ -141,6 +144,11 @@ GList *nicklist_getnicks(CHANNEL_REC *channel)
     return list;
 }
 
+#define isnickchar(a) \
+    (isalnum(a) || (a) == '`' || (a) == '-' || (a) == '_' || \
+    (a) == '[' || (a) == ']' || (a) == '{' || (a) == '}' || \
+    (a) == '|' || (a) == '\\' || (a) == '^')
+
 gchar *nick_strip(gchar *nick)
 {
     gchar *stripped, *spos;
@@ -148,11 +156,13 @@ gchar *nick_strip(gchar *nick)
     g_return_val_if_fail(nick != NULL, NULL);
 
     spos = stripped = g_strdup(nick);
-    while (*nick != '\0' && *nick != ' ' && *nick != ':' && *nick != ',')
+    while (isnickchar(*nick))
     {
-        if (isalnum((gint) *nick) || (guchar) *nick >= 128) *spos++ = *nick;
+        if (isalnum((gint) *nick)) *spos++ = *nick;
         nick++;
     }
+    if ((guchar) *nick > 127)
+	*spos++ = *nick; /* just add it so that nicks won't match.. */
     *spos = '\0';
     return stripped;
 }
@@ -204,7 +214,7 @@ static gboolean event_end_of_names(gchar *data, SERVER_REC *server)
     server->names_coming = FALSE;
 
     chanrec = channel_find(server, channel);
-    if (chanrec != NULL && !chanrec->names_got)
+    if (chanrec != NULL && !chanrec->names_got && chanrec->type == CHANNEL_TYPE_CHANNEL)
     {
 	/* send the query signal.. */
 	chanrec->names_got = TRUE;
@@ -362,11 +372,12 @@ static gboolean event_nick_in_use(gchar *data, SERVER_REC *server)
     }
 
     /* nick already in use - need to change it .. */
-    if (strcmp(server->nick, setup_get_str("default_nick")) == 0)
+    if (strcmp(server->nick, setup_get_str("default_nick")) == 0 &&
+	*setup_get_str("alternate_nick") != '\0')
     {
         /* first try, so try the alternative nick.. */
         g_free(server->nick);
-        server->nick = g_strdup(setup_get_str("alternate_nick"));
+	server->nick = g_strdup(setup_get_str("alternate_nick"));
     }
     else if (strlen(server->nick) < 9)
     {
@@ -430,18 +441,8 @@ static gboolean event_nick(gchar *data, SERVER_REC *server, gchar *orignick)
 
     params = event_get_params(data, 1, &nick);
 
-    if (g_strcasecmp(orignick, server->nick) == 0)
-    {
-        /* You changed your nick */
-        g_free(server->wanted_nick);
-        g_free(server->nick);
-        server->wanted_nick = g_strdup(nick);
-        server->nick = g_strdup(nick);
-        signal_emit("server nick changed", 1, server);
-    }
-
-    /* Someone else changed nick */
-    for (tmp = g_list_first(channels); tmp != NULL; tmp = tmp->next)
+    /* Change nick from nicklists */
+    for (tmp = channels; tmp != NULL; tmp = tmp->next)
     {
         CHANNEL_REC *chanrec = tmp->data;
 
@@ -478,6 +479,17 @@ static gboolean event_nick(gchar *data, SERVER_REC *server, gchar *orignick)
         }
     }
 
+    if (g_strcasecmp(orignick, server->nick) == 0)
+    {
+        /* You changed your nick */
+        g_free(server->wanted_nick);
+        g_free(server->nick);
+        server->wanted_nick = g_strdup(nick);
+        server->nick = g_strdup(nick);
+        signal_emit("server nick changed", 1, server);
+    }
+
+
     g_free(params);
     return TRUE;
 }
@@ -486,6 +498,7 @@ static gboolean event_userhost(gchar *data, SERVER_REC *server)
 {
     gchar *params, *hosts;
     GList *list, *tmp;
+    gboolean ircop;
     gchar *ptr;
 
     g_return_val_if_fail(data != NULL, FALSE);
@@ -497,10 +510,21 @@ static gboolean event_userhost(gchar *data, SERVER_REC *server)
     for (tmp = g_list_first(list); tmp != NULL; tmp = tmp->next)
     {
         ptr = strchr((gchar *) tmp->data, '=');
-        if (ptr == NULL) continue;
-        *ptr++ = '\0';
+	if (ptr == NULL) continue;
 
-        nicklist_update_flags(server, (gchar *) tmp->data, *ptr == '-', -1);
+	if (ptr > (gchar *) tmp->data && ptr[-1] == '*')
+	{
+	    ircop = TRUE;
+	    ptr[-1] = '\0';
+	}
+	else
+	{
+	    ircop = FALSE;
+	    *ptr = '\0';
+	}
+	ptr++;
+
+        nicklist_update_flags(server, (gchar *) tmp->data, *ptr == '-', ircop);
     }
 
     if (list != NULL)
