@@ -22,7 +22,7 @@
 
 GList *servers;
 
-static GList *lookup_servers;
+GList *lookup_servers;
 static gint cmd_tag;
 
 static void server_init(SERVER_REC *server)
@@ -47,6 +47,12 @@ static void server_init(SERVER_REC *server)
     }
 
     server->cmdcount = 0; /* no slowups needed here :) */
+    if (*server->wanted_nick == '\0')
+    {
+	/* empty nick for some reason? */
+	g_free(server->wanted_nick);
+	server->wanted_nick = g_strdup("someone");
+    }
     g_string_sprintf(str, "NICK %s", server->wanted_nick);
     irc_send_cmd(server, str->str);
 
@@ -69,7 +75,7 @@ static gboolean server_remove_channels(SERVER_REC *server)
     g_return_val_if_fail(server != NULL, FALSE);
 
     found = FALSE;
-    for (tmp = g_list_first(channels); tmp != NULL; tmp = next)
+    for (tmp = channels; tmp != NULL; tmp = next)
     {
         channel = tmp->data;
         next = tmp->next;
@@ -80,10 +86,7 @@ static gboolean server_remove_channels(SERVER_REC *server)
 	    if (channel->type != CHANNEL_TYPE_CHANNEL)
 		signal_emit("channel server changed", 1, channel);
 	    else
-	    {
-		channel_destroy(channel);
-		found = TRUE;
-	    }
+                channel_destroy(channel);
         }
     }
 
@@ -96,8 +99,6 @@ static void server_cant_connect(SERVER_REC *server, gchar *msg)
 
     lookup_servers = g_list_remove(lookup_servers, server);
 
-    server->connection_lost = TRUE;
-
     signal_emit("server connect failed", 2, server, msg);
     server_remove_channels(server);
 
@@ -111,8 +112,12 @@ static void server_cant_connect(SERVER_REC *server, gchar *msg)
     }
     if (server->ircnet != NULL) g_free(server->ircnet);
     if (server->password != NULL) g_free(server->password);
+    if (server->autojoin_channels != NULL) g_free(server->autojoin_channels);
     g_free(server->tag);
     g_free(server->address);
+    g_free(server->realname);
+    g_free(server->username);
+    g_free(server->wanted_nick);
     g_free(server->nick);
     g_free(server);
 }
@@ -123,9 +128,33 @@ static gchar *server_create_tag(SERVER_REC *server)
     gchar *start, *end, *tag;
     gint num;
 
-    /* irc.blah.org -> blah */
-    start = strchr(server->address, '.');
-    end = strrchr(server->address, '.');
+    /* prefer the irc network name as tag */
+    if (server->ircnet != NULL)
+    {
+	str = g_string_new(server->ircnet);
+
+	for (num = 2; server_find_tag(str->str) != NULL; num++)
+	    g_string_sprintf(str, "%s%d", server->ircnet, num);
+
+	tag = str->str;
+	g_string_free(str, FALSE);
+	return tag;
+    }
+
+    /* try to generate a reasonable server tag */
+    if (g_strncasecmp(server->address, "irc", 3) == 0 ||
+	g_strncasecmp(server->address, "chat", 4) == 0)
+    {
+	/* irc-2.cs.hut.fi -> cs.hut, chat.bt.net -> bt */
+	start = strchr(server->address, '.');
+	end = strrchr(server->address, '.');
+    }
+    else
+    {
+	/* efnet.cs.hut.fi -> efnet */
+	end = strchr(server->address, '.');
+	start = end;
+    }
     if (start == end) start = server->address; else start++;
     if (end == NULL) end = server->address + strlen(server->address);
 
@@ -215,6 +244,7 @@ static void server_connect_callback_init(SERVER_REC *server, gint handle)
     error = net_geterror(handle);
     if (error != 0)
     {
+	server->connection_lost = TRUE;
 	server_cant_connect(server, strerror(error));
 	return;
     }
@@ -241,11 +271,9 @@ static void server_connect_callback_init(SERVER_REC *server, gint handle)
 
 static void get_source_host_ip(void)
 {
-    IPADDR ip;
-
     /* FIXME: This will block! */
     source_host_ok = *setup_get_str("source_host") != '\0' &&
-	net_gethostname(setup_get_str("source_host"), &ip) != 0;
+	net_gethostname(setup_get_str("source_host"), &source_host_ip) == 0;
 }
 
 void server_connect_callback_readpipe(SERVER_REC *server, gint handle)
@@ -296,6 +324,7 @@ void server_connect_callback_readpipe(SERVER_REC *server, gint handle)
 
     if (!readok)
     {
+	server->connection_lost = TRUE;
 	server_cant_connect(server, "server_connect_callback_readpipe() : internal error (theory != reality), "
 			    "this should never happen, so I don't bother to fix it..");
         return;
@@ -308,6 +337,7 @@ void server_connect_callback_readpipe(SERVER_REC *server, gint handle)
     if (server->handle == -1)
     {
         /* failed */
+	server->connection_lost = TRUE;
 	server_cant_connect(server,
 			    error != -1 ? strerror(errno) : /* connect() failed */
 			    (errormsg != NULL ? errormsg : "Host lookup failed")); /* gethostbyname() failed */
@@ -337,7 +367,7 @@ static IRCNET_REC *ircnet_find(gchar *name)
     return NULL;
 }
 
-SERVER_REC *server_connect(gchar *address, gint port, gchar *password, gchar *nick)
+SERVER_REC *server_connect(gchar *address, gint port, gchar *password, gchar *nick, gchar *channels)
 {
     SETUP_SERVER_REC *srec;
     SERVER_REC *rec;
@@ -355,8 +385,6 @@ SERVER_REC *server_connect(gchar *address, gint port, gchar *password, gchar *ni
         return NULL;
     }
 
-    lookup_servers = g_list_append(lookup_servers, rec);
-
     /* get ircnet/password from setup */
     srec = server_setup_find(address, -1);
     if (srec != NULL)
@@ -409,15 +437,17 @@ SERVER_REC *server_connect(gchar *address, gint port, gchar *password, gchar *ni
     rec->address = g_strdup(address);
     rec->port = port == 0 ? 6667 : port;
     rec->tag = server_create_tag(rec);
+    rec->autojoin_channels = channels == NULL ? NULL : g_strdup(channels);
     rec->handle = -1;
 
     rec->connect_pid =
         net_gethostname_nonblock(setup_get_bool("toggle_use_ircproxy") ? setup_get_str("proxy_address") : rec->address,
                                  rec->connect_pipe[1]);
 
-    rec->connect_tag = gui_input_add(rec->connect_pipe[0], GUI_INPUT_READ/* | GUI_INPUT_EXCEPTION*/,
+    rec->connect_tag = gui_input_add(rec->connect_pipe[0], GUI_INPUT_READ,
                                      (GUIInputFunction) server_connect_callback_readpipe, rec);
 
+    lookup_servers = g_list_append(lookup_servers, rec);
     signal_emit("server looking", 1, rec);
 
     return rec;
@@ -464,9 +494,13 @@ void server_disconnect(SERVER_REC *server)
     if (server->password != NULL) g_free(server->password);
     if (server->usermode != NULL) g_free(server->usermode);
     if (server->real_address != NULL) g_free(server->real_address);
+    if (server->autojoin_channels != NULL) g_free(server->autojoin_channels);
     g_free(server->tag);
-    g_free(server->nick);
     g_free(server->address);
+    g_free(server->realname);
+    g_free(server->username);
+    g_free(server->wanted_nick);
+    g_free(server->nick);
     g_free(server);
 }
 
@@ -490,15 +524,25 @@ SETUP_SERVER_REC *server_setup_find(gchar *address, gint port)
 SERVER_REC *server_find_tag(gchar *tag)
 {
     SERVER_REC *server;
-    GList *servlist;
+    GList *tmp;
 
     g_return_val_if_fail(tag != NULL, NULL);
     if (*tag == '\0') return NULL;
 
-    for (servlist = g_list_first(servers); servlist != NULL; servlist = servlist->next)
+    for (tmp = servers; tmp != NULL; tmp = tmp->next)
     {
-        server = (SERVER_REC *) servlist->data;
-        if (strcmp(server->tag, tag) == 0) return server;
+	server = tmp->data;
+
+	if (strcmp(server->tag, tag) == 0)
+	    return server;
+    }
+
+    for (tmp = lookup_servers; tmp != NULL; tmp = tmp->next)
+    {
+	server = tmp->data;
+
+	if (strcmp(server->tag, tag) == 0)
+	    return server;
     }
 
     return NULL;
@@ -507,33 +551,62 @@ SERVER_REC *server_find_tag(gchar *tag)
 SERVER_REC *server_find_ircnet(gchar *ircnet)
 {
     SERVER_REC *server;
-    GList *servlist;
+    GList *tmp;
 
     g_return_val_if_fail(ircnet != NULL, NULL);
     if (*ircnet == '\0') return NULL;
 
-    for (servlist = g_list_first(servers); servlist != NULL; servlist = servlist->next)
+    for (tmp = servers; tmp != NULL; tmp = tmp->next)
     {
-        server = (SERVER_REC *) servlist->data;
+        server = (SERVER_REC *) tmp->data;
         if (server->ircnet != NULL && strcmp(server->ircnet, ircnet) == 0) return server;
     }
 
     return NULL;
 }
 
-static gboolean event_connected(gchar *data, SERVER_REC *server, gchar *from)
+gchar *server_get_channels(SERVER_REC *server)
 {
-    gchar *params, *ircd_version, *usermodes, *chanmodes, *str;
+    GList *tmp;
+    GString *chans;
+    gchar *ret;
 
     g_return_val_if_fail(server != NULL, FALSE);
 
-    params = event_get_params(data, 5, NULL, NULL, &ircd_version, &usermodes, &chanmodes);
-
-    if (strncmp(ircd_version, "pircd", 5) == 0)
+    chans = g_string_new(NULL);
+    for (tmp = channels; tmp != NULL; tmp = tmp->next)
     {
-	/* yet another stupid irc server that can't understand even the most
-	   simple command: PING. Forget about lag detection with this server. */
-        server->no_lag_check = TRUE;
+	CHANNEL_REC *channel = tmp->data;
+
+	if (channel->server == server &&
+	    channel->type == CHANNEL_TYPE_CHANNEL)
+	{
+	    if (channel->key == NULL)
+		g_string_sprintfa(chans, "%s,", channel->name);
+	    else
+		g_string_sprintfa(chans, "%s %s,", channel->name, channel->key);
+	}
+    }
+
+    ret = chans->str;
+    g_string_free(chans, FALSE);
+
+    return ret;
+}
+
+static gboolean event_connected(gchar *data, SERVER_REC *server, gchar *from)
+{
+    gchar *params, *nick, *str;
+
+    g_return_val_if_fail(server != NULL, FALSE);
+
+    params = event_get_params(data, 1, &nick);
+
+    if (strcmp(server->nick, nick) != 0)
+    {
+	/* nick changed unexpectedly .. connected via proxy, etc. */
+	g_free(server->nick);
+	server->nick = g_strdup(nick);
     }
 
     if (server->real_address == NULL)
@@ -542,10 +615,6 @@ static gboolean event_connected(gchar *data, SERVER_REC *server, gchar *from)
 	server->real_address = g_strdup(from);
     }
 
-    /* check if server understands I and e channel modes */
-    if (strchr(chanmodes, 'I') == NULL || strchr(chanmodes, 'e') == NULL)
-	server->emode_not_known = TRUE;
-
     /* last welcome message found - commands can be sent to server now. */
     server->connected = 1;
 
@@ -560,6 +629,29 @@ static gboolean event_connected(gchar *data, SERVER_REC *server, gchar *from)
     return TRUE;
 }
 
+static gboolean event_server_info(gchar *data, SERVER_REC *server, gchar *from)
+{
+    gchar *params, *ircd_version, *usermodes, *chanmodes;
+
+    g_return_val_if_fail(server != NULL, FALSE);
+
+    params = event_get_params(data, 5, NULL, NULL, &ircd_version, &usermodes, &chanmodes);
+
+    if (strncmp(ircd_version, "pircd", 5) == 0)
+    {
+       /* yet another stupid irc server that can't understand even the most
+          simple command: PING. Forget about lag detection with this server. */
+        server->no_lag_check = TRUE;
+    }
+
+    /* check if server understands I and e channel modes */
+    if (strchr(chanmodes, 'I') == NULL || strchr(chanmodes, 'e') == NULL)
+       server->emode_not_known = TRUE;
+
+    g_free(params);
+    return TRUE;
+}
+
 static gboolean event_ping(gchar *data, SERVER_REC *server)
 {
     gchar *str;
@@ -589,7 +681,8 @@ void servers_init(void)
     lookup_servers = servers = NULL;
 
     signal_add("startup settings read", (SIGNAL_FUNC) sig_config_read);
-    signal_add("event 004", (SIGNAL_FUNC) event_connected);
+    signal_add("event 001", (SIGNAL_FUNC) event_connected);
+    signal_add("event 004", (SIGNAL_FUNC) event_server_info);
     signal_add("event ping", (SIGNAL_FUNC) event_ping);
     signal_add("event empty", (SIGNAL_FUNC) event_empty);
 }
@@ -605,7 +698,8 @@ void servers_deinit(void)
     g_list_free(lookup_servers);
 
     signal_remove("startup settings read", (SIGNAL_FUNC) sig_config_read);
-    signal_remove("event 004", (SIGNAL_FUNC) event_connected);
+    signal_remove("event 001", (SIGNAL_FUNC) event_connected);
+    signal_remove("event 004", (SIGNAL_FUNC) event_server_info);
     signal_remove("event ping", (SIGNAL_FUNC) event_ping);
     signal_remove("event empty", (SIGNAL_FUNC) event_empty);
 }
