@@ -31,6 +31,7 @@ void irc_extra_init(void)
     ignore_init();
     log_init();
     notifylist_init();
+    irssi_perl_init();
     plugins_init();
     plugins_setup_init();
     plugin_commands_init();
@@ -46,6 +47,7 @@ void irc_extra_deinit(void)
     ignore_deinit();
     log_deinit();
     notifylist_deinit();
+    irssi_perl_deinit();
     plugins_deinit();
     plugins_setup_deinit();
     plugin_commands_deinit();
