@@ -34,6 +34,7 @@ OPTCATEGORY_REC option_categories[] =
     { "server", server_options }
 };
 
+gboolean readonly = FALSE;
 static GHashTable *settings;
 
 void setup_themes_init(void);
@@ -64,11 +65,13 @@ static gchar *url_mail_client;
 static gchar *ctcp_version_reply;
 static gchar *default_quit_message;
 static gchar *default_user_mode;
+static gchar *translation;
 
 static gint min_lag_check_time;
 static gint max_lag_before_disconnect;
 
 static gint knockout_time; /* How many seconds to keep /knockouted ban */
+static gint timestamp_timeout; /* How many seconds idle before show timestamp */
 
 static gchar *awaylog_file;
 static gchar *awaylog_level;
@@ -82,14 +85,16 @@ SETUP_REC misc_options[] =
     { SETUP_TYPE_ENTRY, &ctcp_version_reply, "ctcp_version_reply", PACKAGE" v%s - running on %s %s" },
     { SETUP_TYPE_ENTRY, &default_quit_message, "default_quit_message", "leaving" },
     { SETUP_TYPE_ENTRY, &default_user_mode, "default_user_mode", "+iw" },
+    { SETUP_TYPE_ENTRY, &translation, "translation", "" },
 
-    { SETUP_TYPE_SPIN, &min_lag_check_time, "min_lag_check_time", GINT_TO_POINTER(10) },
-    { SETUP_TYPE_SPIN, &max_lag_before_disconnect, "max_lag_before_disconnect", GINT_TO_POINTER(90) },
+    { SETUP_TYPE_SPIN, &min_lag_check_time, "min_lag_check_time", GINT_TO_POINTER(20) },
+    { SETUP_TYPE_SPIN, &max_lag_before_disconnect, "max_lag_before_disconnect", GINT_TO_POINTER(300) },
 
     { SETUP_TYPE_SPIN, &knockout_time, "knockout_time", GINT_TO_POINTER(300) },
 
     { SETUP_TYPE_ENTRY, &awaylog_file, "awaylog_file", "~/.irssi/away.log" },
     { SETUP_TYPE_ENTRY, &awaylog_level, "awaylog_level", "-all +msgs +hilight" },
+    { SETUP_TYPE_SPIN, &timestamp_timeout, "timestamp_timeout", GINT_TO_POINTER(0) },
 
     { 0, NULL, NULL }
 };
@@ -126,8 +131,8 @@ SETUP_REC completion_options[] =
 {
     { SETUP_TYPE_ENTRY, &completion_char, "completion_char", ":" },
     { SETUP_TYPE_TOGGLEBUTTON, &completion_disable_auto, "completion_disable_auto", GINT_TO_POINTER(FALSE) },
-    { SETUP_TYPE_SPIN, &completion_keep_publics, "completion_keep_publics", GINT_TO_POINTER(30) },
-    { SETUP_TYPE_SPIN, &completion_keep_ownpublics, "completion_keep_ownpublics", GINT_TO_POINTER(60) },
+    { SETUP_TYPE_SPIN, &completion_keep_publics, "completion_keep_publics", GINT_TO_POINTER(180) },
+    { SETUP_TYPE_SPIN, &completion_keep_ownpublics, "completion_keep_ownpublics", GINT_TO_POINTER(360) },
     { SETUP_TYPE_SPIN, &completion_keep_privates, "completion_keep_privates", GINT_TO_POINTER(10) },
 
     { 0, NULL, NULL }
@@ -164,6 +169,8 @@ static gboolean toggle_hide_text_style;
 static gboolean toggle_bell_beeps;
 static gboolean toggle_actlist_moves;
 static gboolean toggle_privmsg_beeps;
+static gboolean toggle_show_nickmode;
+static gboolean toggle_show_topicbar;
 
 static gboolean toggle_use_status_window;
 static gboolean toggle_use_msgs_window;
@@ -173,6 +180,8 @@ static gboolean toggle_notifylist_popups;
 static gboolean toggle_use_tabbed_windows;
 static gint tab_orientation;
 
+static gboolean toggle_use_colors;
+
 SETUP_REC lookandfeel_options[] =
 {
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_show_menubar, "toggle_show_menubar", GINT_TO_POINTER(TRUE) },
@@ -185,6 +194,8 @@ SETUP_REC lookandfeel_options[] =
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_bell_beeps, "toggle_bell_beeps", GINT_TO_POINTER(FALSE) },
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_actlist_moves, "toggle_actlist_moves", GINT_TO_POINTER(FALSE) },
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_privmsg_beeps, "toggle_privmsg_beeps", GINT_TO_POINTER(FALSE) },
+    { SETUP_TYPE_TOGGLEBUTTON, &toggle_show_nickmode, "toggle_show_nickmode", GINT_TO_POINTER(TRUE) },
+    { SETUP_TYPE_TOGGLEBUTTON, &toggle_show_topicbar, "toggle_show_topicbar", GINT_TO_POINTER(TRUE) },
 
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_use_status_window, "toggle_use_status_window", GINT_TO_POINTER(FALSE) },
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_use_msgs_window, "toggle_use_msgs_window", GINT_TO_POINTER(TRUE) },
@@ -194,6 +205,8 @@ SETUP_REC lookandfeel_options[] =
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_use_tabbed_windows, "toggle_use_tabbed_windows", GINT_TO_POINTER(TRUE) },
     { SETUP_TYPE_INT_OBJECT,   &tab_orientation, "tab_orientation", GINT_TO_POINTER(3) },
 
+    { SETUP_TYPE_TOGGLEBUTTON, &toggle_use_colors, "colors", GINT_TO_POINTER(TRUE) },
+
     { 0, NULL, NULL }
 };
 
@@ -256,8 +269,10 @@ SETUP_REC dcc_options[] =
 /* appearance */
 static gboolean toggle_buggy_gtkthemes;
 static gboolean toggle_use_itext;
+static gboolean toggle_use_fontset;
 static gboolean toggle_background_transparent;
 
+static gboolean toggle_use_panel;
 static gint panel_max_channels;
 
 SETUP_REC appearance_options[] =
@@ -265,12 +280,28 @@ SETUP_REC appearance_options[] =
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_buggy_gtkthemes, "toggle_buggy_gtkthemes", GINT_TO_POINTER(FALSE) },
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_use_itext, "toggle_use_itext", GINT_TO_POINTER(TRUE) },
     { SETUP_TYPE_TOGGLEBUTTON, &toggle_background_transparent, "toggle_background_transparent", GINT_TO_POINTER(FALSE) },
+    { SETUP_TYPE_TOGGLEBUTTON, &toggle_use_fontset, "toggle_use_fontset", GINT_TO_POINTER(FALSE) },
 
+    { SETUP_TYPE_TOGGLEBUTTON, &toggle_use_panel, "toggle_use_panel", GINT_TO_POINTER(TRUE) },
     { SETUP_TYPE_SPIN,         &panel_max_channels, "panel_max_channels", GINT_TO_POINTER(-1) },
 
     { 0, NULL, NULL }
 };
 
+SETUP_REC *all_options[] =
+{
+    misc_options,
+    history_options,
+    flood_options,
+    completion_options,
+    lookandfeel_options,
+    ircproxy_options,
+    dcc_options,
+    appearance_options,
+    server_options,
+    NULL
+};
+
 void setup_set_data(SETUP_REC *rec, gchar *section, gchar *value)
 {
     proplist_t sectprop, pkey, pvalue;
@@ -397,11 +428,40 @@ void setup_free(SETUP_REC *options)
     }
 }
 
+static gboolean sig_readonly(void)
+{
+    signal_emit("gui dialog", 2, "warning",
+		"Another Irssi is running - no settings can be saved "
+		"and no log files can be created. (If you're sure there's no "
+		"other irssi running, you can get rid of this by removing ~/.irssi/lock file)");
+    signal_remove("irssi init finished", (SIGNAL_FUNC) sig_readonly);
+    return TRUE;
+}
+
+void sig_term(int n)
+{
+    gchar *str;
+
+    signal(SIGTERM, SIG_DFL);
+
+    /* remove the lock file */
+    str = g_strdup_printf("%s/.irssi/lock", g_get_home_dir());
+    remove(str);
+    g_free(str);
+
+    /* quit from all servers too.. */
+    signal_emit("command quit", 1, "");
+
+    /* and die */
+    kill(getpid(), SIGTERM);
+}
+
 void init_configfile(void)
 {
     struct stat statbuf;
     proplist_t fname;
-    gchar *config_file, *str;
+    gchar *config_file, *str, buf[256];
+    FILE *f;
 
     str = g_strdup_printf("%s/.irssi", g_get_home_dir());
     if (stat(str, &statbuf) != 0)
@@ -444,6 +504,55 @@ void init_configfile(void)
 	PLRelease(fname);
     }
     g_free(config_file);
+
+    /* remove lock file in SIGTERM */
+    signal(SIGTERM, sig_term);
+
+    /* check lock file .. we have a small race condition here
+       but I don't think that will be a problem :) */
+    str = g_strdup_printf("%s/.irssi/lock", g_get_home_dir());
+    f = fopen(str, "r");
+
+    readonly = FALSE;
+    if (f != NULL)
+    {
+	/* get pid */
+	if (fgets(buf, sizeof(buf), f) != NULL)
+	{
+	    /* check if pid exists */
+	    if (kill(atol(buf), 0) == 0 || errno == EPERM)
+	    {
+		/* we're locked .. */
+		readonly = TRUE;
+		signal_add("irssi init finished", (SIGNAL_FUNC) sig_readonly);
+	    }
+	}
+        fclose(f);
+    }
+
+    if (!readonly)
+    {
+	/* create the lock file */
+	f = fopen(str, "w");
+	fprintf(f, "%d", getpid());
+	fclose(f);
+    }
+    g_free(str);
+}
+
+gint setup_option_type(gchar *option)
+{
+    SETUP_REC **rec;
+    gint n;
+
+    for (rec = all_options; *rec != NULL; rec++)
+    {
+	for (n = 0; (*rec)[n].data != NULL; n++)
+	    if (g_strcasecmp((*rec)[n].tag, option) == 0)
+		return (*rec)[n].type;
+    }
+
+    return -1;
 }
 
 gchar *setup_get_str(gchar *option)
@@ -549,4 +658,14 @@ void settings_deinit(void)
     if (cprop != NULL) PLRelease(cprop);
 
     g_hash_table_destroy(settings);
+
+    if (!readonly)
+    {
+	/* remove the lock file */
+	gchar *str;
+
+	str = g_strdup_printf("%s/.irssi/lock", g_get_home_dir());
+	remove(str);
+	g_free(str);
+    }
 }
