@@ -20,6 +20,22 @@
 
 #include "irssi.h"
 
+static gchar *ret_texts[] =
+{
+    "Invalid parameter",
+    "Not enough parameters given",
+    "Not connected to IRC server yet",
+    "Not joined to any channels yet",
+    "Error: getsockname() failed",
+    "Error: listen() failed",
+    "Multiple matches found, be more specific",
+    "Nick not found",
+    "Not joined to such channel",
+    "Server not found",
+    "Channel not fully synchronized yet, try again after a while",
+    "Doing this is not a good idea. Add -YES if you really mean it",
+};
+
 static GString *tmpstr;
 
 static gint commands_compare(COMMAND_REC *rec, COMMAND_REC *rec2)
@@ -191,14 +207,41 @@ static gboolean cmd_help(gchar *data)
 
 static gboolean cmd_servers(gchar *data)
 {
-    SERVER_REC *server;
-    GList *servlist;
+    GList *tmp;
+    gchar *tag, *next_connect;
+    gint left;
 
-    for (servlist = g_list_first(servers); servlist != NULL; servlist = servlist->next)
+    for (tmp = servers; tmp != NULL; tmp = tmp->next)
     {
-        server = (SERVER_REC *) servlist->data;
+        SERVER_REC *rec = tmp->data;
 
-        printtext(server, NULL, MSGLEVEL_CRAP, "%_%s%_: %s:%d", server->tag, server->address, server->port);
+	printformat(NULL, NULL, MSGLEVEL_CRAP, IRCTXT_SERVER_LIST,
+		    rec->tag, rec->address, rec->port,
+		    rec->ircnet == NULL ? "" : rec->ircnet, rec->nick);
+    }
+
+    for (tmp = lookup_servers; tmp != NULL; tmp = tmp->next)
+    {
+        SERVER_REC *rec = tmp->data;
+
+	printformat(NULL, NULL, MSGLEVEL_CRAP, IRCTXT_SERVER_LOOKUP_LIST,
+		    rec->tag, rec->address, rec->port,
+		    rec->ircnet == NULL ? "" : rec->ircnet, rec->nick);
+    }
+
+    for (tmp = reconnects; tmp != NULL; tmp = tmp->next)
+    {
+        RECONNECT_REC *rec = tmp->data;
+
+	tag = g_strdup_printf("RECON-%d", rec->tag);
+	left = rec->next_connect-time(NULL);
+	next_connect = g_strdup_printf("%02d:%02d", left/60, left%60);
+	printformat(NULL, NULL, MSGLEVEL_CRAP, IRCTXT_SERVER_RECONNECT_LIST,
+		    tag, rec->address, rec->port,
+		    rec->ircnet == NULL ? "" : rec->ircnet,
+		    rec->nick, next_connect);
+	g_free(next_connect);
+	g_free(tag);
     }
 
     return TRUE;
@@ -259,14 +302,15 @@ static gboolean cmd_query(gchar *data, SERVER_REC *server, CHANNEL_REC *cur_chan
         return TRUE;
     }
 
-    channel_create(server, data, CHANNEL_TYPE_QUERY);
+    channel_create(server, data, CHANNEL_TYPE_QUERY, FALSE);
     return TRUE;
 }
 
 static gboolean cmd_msg(gchar *data, SERVER_REC *server)
 {
     CHANNEL_REC *channel;
-    gchar *params, *target, *msg;
+    NICK_REC *nickrec;
+    gchar *params, *target, *msg, *nickmode;
 
     g_return_val_if_fail(data != NULL, FALSE);
 
@@ -288,16 +332,20 @@ static gboolean cmd_msg(gchar *data, SERVER_REC *server)
 
     if (ischannel(*target))
     {
-        /* msg to channel */
+	/* msg to channel */
+	nickrec = channel == NULL ? NULL : nicklist_find(channel, server->nick);
+	nickmode = !setup_get_bool("toggle_show_nickmode") || nickrec == NULL ? "" :
+	    nickrec->op ? "@" : nickrec->voice ? "+" : " ";
+
 	if (channel != NULL && CHANNEL_PARENT(channel)->active == channel)
 	{
 	    printformat(server, target, MSGLEVEL_PUBLIC | MSGLEVEL_NOHILIGHT,
-			IRCTXT_OWN_MSG, server->nick, msg);
+			IRCTXT_OWN_MSG, server->nick, msg, nickmode);
 	}
 	else
 	{
 	    printformat(server, target, MSGLEVEL_PUBLIC | MSGLEVEL_NOHILIGHT,
-			IRCTXT_OWN_MSG_CHANNEL, server->nick, target, msg);
+			IRCTXT_OWN_MSG_CHANNEL, server->nick, target, msg, nickmode);
 	}
     }
     else
@@ -343,7 +391,9 @@ static gboolean cmd_me(gchar *data, SERVER_REC *server, CHANNEL_REC *cur_channel
 
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
 
-    printformat(server, cur_channel->name, MSGLEVEL_ACTIONS, IRCTXT_OWN_ME, server->nick, data);
+    printformat(server, cur_channel->name, MSGLEVEL_ACTIONS,
+		IRCTXT_OWN_ME, server->nick, data);
+
     if (cur_channel->type == CHANNEL_TYPE_CHANNEL ||
         cur_channel->type == CHANNEL_TYPE_QUERY)
     {
@@ -354,29 +404,26 @@ static gboolean cmd_me(gchar *data, SERVER_REC *server, CHANNEL_REC *cur_channel
     return TRUE;
 }
 
-static gboolean cmd_topic(gchar *data, SERVER_REC *server, CHANNEL_REC *cur_channel)
+static gboolean cmd_action(gchar *data, SERVER_REC *server)
 {
-    gchar *params, *channel, *topic;
+    gchar *params, *target, *text;
 
     g_return_val_if_fail(data != NULL, FALSE);
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
-
-    params = cmd_get_params(data, 2 | PARAM_FLAG_OPTCHAN | PARAM_FLAG_GETREST, cur_channel, &channel, &topic);
-
-    if (*topic == '\0')
+    if (*data == '=')
     {
-        /* Display topic */
-        CHANNEL_REC *chanrec;
-
-        chanrec = channel_find(server, channel);
-        if (chanrec == NULL) cmd_param_error(CMDERR_CHAN_NOT_FOUND);
-
-        if (chanrec->topic != NULL)
-            printformat(server, channel, MSGLEVEL_TOPICS, IRCTXT_TOPIC, chanrec->name, chanrec->topic);
-        else
-            printformat(server, channel, MSGLEVEL_TOPICS, IRCTXT_NO_TOPIC, chanrec->name);
+	/* DCC action - handled by ui-dcc.c */
+	return TRUE;
     }
 
+    params = cmd_get_params(data, 3 | PARAM_FLAG_GETREST, &target, &text);
+    if (*target == '\0' || *text == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
+
+    printformat(server, target, MSGLEVEL_ACTIONS, IRCTXT_OWN_ME, server->nick, text);
+
+    g_string_sprintf(tmpstr, "PRIVMSG %s :\001ACTION %s\001", target, text);
+    irc_send_cmd(server, tmpstr->str);
+
     g_free(params);
     return TRUE;
 }
@@ -389,7 +436,7 @@ static gboolean cmd_ctcp(gchar *data, SERVER_REC *server)
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
 
     params = cmd_get_params(data, 3 | PARAM_FLAG_GETREST, &target, &ctcpcmd, &ctcpdata);
-    if (*target == '\0' || *ctcpcmd == '\0') cmd_return_error(CMDERR_NOT_ENOUGH_PARAMS);
+    if (*target == '\0' || *ctcpcmd == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
     if (*target == '@' && ischannel(target[1]))
 	target++; /* Hybrid 6 feature, send ctcp to all ops in channel */
@@ -409,7 +456,7 @@ static gboolean cmd_nctcp(gchar *data, SERVER_REC *server)
     if (server == NULL || !server->connected) cmd_return_error(CMDERR_NOT_CONNECTED);
 
     params = cmd_get_params(data, 3 | PARAM_FLAG_GETREST, &target, &ctcpcmd, &ctcpdata);
-    if (*target == '\0' || *ctcpcmd == '\0') cmd_return_error(CMDERR_NOT_ENOUGH_PARAMS);
+    if (*target == '\0' || *ctcpcmd == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
     if (*target == '@' && ischannel(target[1]))
 	target++; /* Hybrid 6 feature, send notice to all ops in channel */
@@ -681,23 +728,38 @@ static gboolean cmd_ver(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
     return TRUE;
 }
 
-/*static gboolean cmd_default(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
+static gboolean cmd_ts(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
 {
-    gchar *cmd, *ptr;
+    GList *tmp;
 
-    cmd = g_strdup(data+1);
-    ptr = strchr(cmd, ' ');
-    if (ptr != NULL) *ptr = '\0';
-    g_strup(cmd);
+    g_return_val_if_fail(data != NULL, FALSE);
 
-    printtext(server, channel == NULL ? NULL : channel->name, MSGLEVEL_CRAP, "Unknown command: %s", cmd);
-    g_free(cmd);
+    for (tmp = channels; tmp != NULL; tmp = tmp->next)
+    {
+	CHANNEL_REC *rec = tmp->data;
+
+	if (rec->type != CHANNEL_TYPE_CHANNEL)
+	    continue;
+
+	printformat(NULL, NULL, MSGLEVEL_CLIENTNOTICE, IRCTXT_TOPIC,
+		    rec->name, rec->topic == NULL ? "" : rec->topic);
+    }
+
+    return TRUE;
+}
+
+static gboolean cmd_unknown(gchar *data, SERVER_REC *server, CHANNEL_REC *channel)
+{
+    data = g_strdup(data); g_strup(data);
+    printtext(server, channel == NULL ? NULL : channel->name, MSGLEVEL_CRAP, "Unknown command: %s", data);
+    g_free(data);
     return FALSE;
-}*/
+}
 
-void ui_command_show_error(gint error)
+static gboolean event_cmderror(gpointer error)
 {
-    printtext(NULL, NULL, MSGLEVEL_CLIENTERROR, "error %d", error);
+    printtext(NULL, NULL, MSGLEVEL_CLIENTERROR, ret_texts[GPOINTER_TO_INT(error)]);
+    return TRUE;
 }
 
 void ui_commands_init(void)
@@ -711,7 +773,7 @@ void ui_commands_init(void)
     command_bind("msg", NULL, (SIGNAL_FUNC) cmd_msg);
     command_bind("notice", NULL, (SIGNAL_FUNC) cmd_notice);
     command_bind("me", NULL, (SIGNAL_FUNC) cmd_me);
-    command_bind("topic", NULL, (SIGNAL_FUNC) cmd_topic);
+    command_bind("action", NULL, (SIGNAL_FUNC) cmd_action);
     command_bind("ctcp", NULL, (SIGNAL_FUNC) cmd_ctcp);
     command_bind("nctcp", NULL, (SIGNAL_FUNC) cmd_nctcp);
     command_bind("banstat", NULL, (SIGNAL_FUNC) cmd_banstat);
@@ -721,8 +783,11 @@ void ui_commands_init(void)
     command_bind("version", NULL, (SIGNAL_FUNC) cmd_version);
     command_bind("sv", NULL, (SIGNAL_FUNC) cmd_sv);
     command_bind("ver", NULL, (SIGNAL_FUNC) cmd_ver);
+    command_bind("ts", NULL, (SIGNAL_FUNC) cmd_ts);
 
-    /*signal_add("default command", (SIGNAL_FUNC) cmd_default);*/
+    signal_add("unknown command", (SIGNAL_FUNC) cmd_unknown);
+    signal_add("error command", (SIGNAL_FUNC) event_cmderror);
+    /*signal_add("default command", (SIGNAL_FUNC) cmd_unknown);*/
 }
 
 void ui_commands_deinit(void)
@@ -734,7 +799,7 @@ void ui_commands_deinit(void)
     command_unbind("msg", (SIGNAL_FUNC) cmd_msg);
     command_unbind("notice", (SIGNAL_FUNC) cmd_notice);
     command_unbind("me", (SIGNAL_FUNC) cmd_me);
-    command_unbind("topic", (SIGNAL_FUNC) cmd_topic);
+    command_unbind("action", (SIGNAL_FUNC) cmd_action);
     command_unbind("ctcp", (SIGNAL_FUNC) cmd_ctcp);
     command_unbind("nctcp", (SIGNAL_FUNC) cmd_nctcp);
     command_unbind("banstat", (SIGNAL_FUNC) cmd_banstat);
@@ -744,8 +809,11 @@ void ui_commands_deinit(void)
     command_unbind("version", (SIGNAL_FUNC) cmd_version);
     command_unbind("sv", (SIGNAL_FUNC) cmd_sv);
     command_unbind("ver", (SIGNAL_FUNC) cmd_ver);
+    command_unbind("ts", (SIGNAL_FUNC) cmd_ts);
 
-    /*signal_remove("default command", (SIGNAL_FUNC) cmd_default);*/
+    signal_remove("unknown command", (SIGNAL_FUNC) cmd_unknown);
+    signal_remove("error command", (SIGNAL_FUNC) event_cmderror);
+    /*signal_remove("default command", (SIGNAL_FUNC) cmd_unknown);*/
 
     g_string_free(tmpstr, TRUE);
 }
