@@ -53,46 +53,108 @@ static gboolean event_names_list(gchar *data, SERVER_REC *server)
     g_return_val_if_fail(data != NULL, FALSE);
 
     params = event_get_params(data, 4, NULL, NULL, &channel, &names);
-    printformat(server, channel, MSGLEVEL_CRAP, IRCTXT_NAMES, channel, names);
+    if (channel_find(server, channel) == NULL)
+	printformat(server, channel, MSGLEVEL_CRAP, IRCTXT_NAMES, channel, names);
     g_free(params);
     return TRUE;
 }
 
+static void display_sorted_nicks(CHANNEL_REC *channel, GList *nicklist, gint items, gint max)
+{
+    NICK_REC *rec, *last;
+    GString *str;
+    GList *tmp;
+    gint lines, cols, line, col, skip;
+    gchar *linebuf;
+
+    max++; /* op/voice */
+    str = g_string_new(NULL);
+
+    cols = max > 70 ? 1 : (70 / (max+3)); /* "[] " */
+    lines = items <= cols ? 1 : items / cols+1;
+
+    last = NULL; linebuf = g_malloc(max+1); linebuf[max] = '\0';
+    for (line = 0, col = 0, skip = 1, tmp = nicklist; line < lines; last = rec, tmp = tmp->next)
+    {
+	rec = tmp->data;
+
+	if (--skip == 0)
+	{
+	    skip = lines;
+	    memset(linebuf, ' ', max);
+	    linebuf[0] = rec->op ? '@' : rec->voice ? '+' : ' ';
+	    memcpy(linebuf+1, rec->nick, strlen(rec->nick));
+	    g_string_sprintfa(str, "%%K[%%n%%_%c%%_%s%%K] ", linebuf[0], linebuf+1);
+	    cols++;
+	}
+
+	if (col == cols || tmp->next == NULL)
+	{
+	    printtext(channel->server, channel->name, MSGLEVEL_CLIENTCRAP, str->str);
+	    g_string_truncate(str, 0);
+	    col = 0; line++;
+	    tmp = g_list_nth(nicklist, line-1); skip = 1;
+	}
+    }
+    if (str->len != 0)
+	printtext(channel->server, channel->name, MSGLEVEL_CLIENTCRAP, str->str);
+    g_string_free(str, TRUE);
+    g_free(linebuf);
+}
+
+static void display_nicks(CHANNEL_REC *channel)
+{
+    NICK_REC *nick;
+    GList *tmp, *nicklist, *sorted;
+    gint nicks, normal, voices, ops, len, max;
+
+    nicks = normal = voices = ops = 0;
+    nicklist = nicklist_getnicks(channel);
+    sorted = NULL;
+
+    /* sort the nicklist */
+    max = 0;
+    for (tmp = nicklist; tmp != NULL; tmp = tmp->next)
+    {
+	nick = tmp->data;
+
+	sorted = g_list_insert_sorted(sorted, nick, (GCompareFunc) nicklist_compare);
+        if (nick->op)
+	    ops++;
+	else if (nick->voice)
+	    voices++;
+	else
+	    normal++;
+	nicks++;
+
+	len = strlen(nick->nick);
+	if (len > max) max = len;
+    }
+    g_list_free(nicklist);
+
+    /* display the nicks */
+    printformat(channel->server, channel->name, MSGLEVEL_CRAP, IRCTXT_NAMES, channel->name, "");
+    display_sorted_nicks(channel, sorted, nicks, max);
+    g_list_free(sorted);
+
+    printformat(channel->server, channel->name, MSGLEVEL_CRAP, IRCTXT_ENDOFNAMES,
+		channel->name, nicks, ops, voices, normal);
+}
+
 static gboolean event_end_of_names(gchar *data, SERVER_REC *server)
 {
     gchar *params, *channel;
     CHANNEL_REC *chanrec;
-    gint nicks, normal, voices, ops;
-    GList *tmp, *nicklist;
 
     g_return_val_if_fail(data != NULL, FALSE);
 
     params = event_get_params(data, 2, NULL, &channel);
 
     chanrec = channel_find(server, channel);
-    if (chanrec == NULL)
+    if (chanrec == NULL || chanrec->type != CHANNEL_TYPE_CHANNEL)
         printformat(server, channel, MSGLEVEL_CRAP, IRCTXT_ENDOFNAMES, channel, 0, 0, 0, 0);
     else
-    {
-	nicks = normal = voices = ops = 0;
-	nicklist = nicklist_getnicks(chanrec);
-        for (tmp = nicklist; tmp != NULL; tmp = tmp->next)
-        {
-            NICK_REC *rec = tmp->data;
-
-            if (rec->op)
-                ops++;
-            else if (rec->voice)
-                voices++;
-            else
-                normal++;
-            nicks++;
-	}
-	g_list_free(nicklist);
-
-        printformat(server, channel, MSGLEVEL_CRAP, IRCTXT_ENDOFNAMES, channel,
-                    nicks, ops, voices, normal);
-    }
+	display_nicks(chanrec);
     g_free(params);
     return TRUE;
 }
@@ -125,7 +187,7 @@ static gboolean event_end_of_who(gchar *data, SERVER_REC *server)
     g_return_val_if_fail(data != NULL, FALSE);
 
     params = event_get_params(data, 2, NULL, &channel);
-    printformat(server, channel, MSGLEVEL_CRAP, IRCTXT_END_OF_WHO);
+    printformat(server, NULL, MSGLEVEL_CRAP, IRCTXT_END_OF_WHO, channel);
     g_free(params);
     return TRUE;
 }
