@@ -20,26 +20,10 @@
 
 #include "irssi.h"
 
-static gchar *ret_texts[] =
-{
-    "Invalid parameter",
-    "Not enough parameters given",
-    "Not connected to IRC server yet",
-    "Not joined to any channels yet",
-    "Error: getsockname() failed",
-    "Error: listen() failed",
-    "Multiple matches found, be more specific",
-    "Nick not found",
-    "Not joined to such channel",
-    "Server not found",
-    "Channel not fully synchronized yet, try again after a while",
-    "Doing this is not a good idea. Add -YES if yoou really mean it",
-};
-
 static gboolean event_privmsg(gchar *data, SERVER_REC *server, gchar *nick, gchar *addr)
 {
     CHANNEL_REC *chanrec;
-    gchar *params, *target, *msg;
+    gchar *params, *target, *msg, *nickmode;
 
     g_return_val_if_fail(data != NULL, FALSE);
 
@@ -57,31 +41,58 @@ static gboolean event_privmsg(gchar *data, SERVER_REC *server, gchar *nick, gcha
     }
     else if (ischannel(*target))
     {
-        /* message to some channel */
-        gboolean toyou;
+	/* message to some channel */
+	NICK_REC *nickrec;
+	gboolean toyou;
+	gchar *color;
 
 	chanrec = channel_find(server, target);
-        toyou = ui_completion_msgtoyou(server, msg);
+	toyou = ui_completion_msgtoyou(server, msg);
+	color = hilight_find_nick(target, nick, addr);
+
+	nickrec = chanrec == NULL ? NULL : nicklist_find(chanrec, nick);
+	nickmode = !setup_get_bool("toggle_show_nickmode") || nickrec == NULL ? "" :
+	    nickrec->op ? "@" : nickrec->voice ? "+" : " ";
 
         if (chanrec != NULL && CHANNEL_PARENT(chanrec)->active == chanrec)
-        {
+	{
             /* message to active channel in window */
-	    printformat(server, target, MSGLEVEL_PUBLIC | (toyou ? MSGLEVEL_NOHILIGHT : 0),
-                        toyou ? IRCTXT_PUBMSG_ME : IRCTXT_PUBMSG, nick, msg);
+	    if (color != NULL)
+	    {
+		/* highlighted nick */
+		printformat(server, target, MSGLEVEL_PUBLIC | MSGLEVEL_NOHILIGHT,
+			    IRCTXT_PUBMSG_HILIGHT, color, nick, msg, nickmode);
+	    }
+	    else
+	    {
+		printformat(server, target, MSGLEVEL_PUBLIC | (toyou ? MSGLEVEL_NOHILIGHT : 0),
+			    toyou ? IRCTXT_PUBMSG_ME : IRCTXT_PUBMSG, nick, msg, nickmode);
+	    }
         }
         else
         {
             /* message to not existing/active channel */
-            printformat(server, target, MSGLEVEL_PUBLIC | (toyou ? MSGLEVEL_NOHILIGHT : 0),
-                        toyou ? IRCTXT_PUBMSG_ME_CHANNEL : IRCTXT_PUBMSG_CHANNEL,
-                        nick, target, msg);
-        }
+	    if (color != NULL)
+	    {
+		/* highlighted nick */
+		printformat(server, target, MSGLEVEL_PUBLIC | MSGLEVEL_NOHILIGHT,
+			    IRCTXT_PUBMSG_HILIGHT_CHANNEL, color, nick, target, msg, nickmode);
+	    }
+	    else
+	    {
+		printformat(server, target, MSGLEVEL_PUBLIC | (toyou ? MSGLEVEL_NOHILIGHT : 0),
+			    toyou ? IRCTXT_PUBMSG_ME_CHANNEL : IRCTXT_PUBMSG_CHANNEL,
+			    nick, target, msg, nickmode);
+	    }
+	}
+
+	if (color != NULL) g_free(color);
     }
     else
     {
         /* private message */
         if (setup_get_bool("toggle_autocreate_query") && channel_find(server, nick) == NULL)
-	    chanrec = channel_create(server, nick, CHANNEL_TYPE_QUERY);
+	    chanrec = channel_create(server, nick, CHANNEL_TYPE_QUERY, TRUE);
 	else
 	    chanrec = channel_find(server, nick);
 	if (setup_get_bool("toggle_privmsg_beeps"))
@@ -114,12 +125,14 @@ static gboolean event_notice(gchar *data, SERVER_REC *server, gchar *nick, gchar
         if (nick == NULL || !ignore_check(server, nick, "", MSGLEVEL_SNOTES))
 	    printformat(server, target, MSGLEVEL_SNOTES, IRCTXT_NOTICE_SERVER, nick == NULL ? "" : nick, msg);
     }
-    else if (ischannel(*target))
+    else if (ischannel(*target) || (*target == '@' && ischannel(target[1])))
     {
         /* notice in some channel */
 	if (!ignore_check(server, nick, addr, MSGLEVEL_NOTICES) &&
 	    !ignore_check(server, target, NULL, MSGLEVEL_NOTICES))
-	    printformat(server, target, MSGLEVEL_NOTICES, IRCTXT_NOTICE_PUBLIC, nick, target, msg);
+	    printformat(server, target, MSGLEVEL_NOTICES,
+			*target == '@' ? IRCTXT_NOTICE_PUBLIC_OPS : IRCTXT_NOTICE_PUBLIC,
+			nick, *target == '@' ? target+1 : target, msg);
     }
     else
     {
@@ -415,7 +428,8 @@ static gboolean event_connected(SERVER_REC *server)
 
     g_return_val_if_fail(server != NULL, FALSE);
 
-    if (g_strcasecmp(server->nick, setup_get_str("default_nick")) == 0)
+    if (*setup_get_str("default_nick") == '\0' ||
+	g_strcasecmp(server->nick, setup_get_str("default_nick")) == 0)
         return TRUE;
 
     /* someone has our nick, find out who. */
@@ -534,9 +548,9 @@ static gboolean event_plugin_not_loaded(gchar *plugin)
     return TRUE;
 }
 
-static gboolean event_cmderror(gpointer error)
+static gboolean event_perl_error(gchar *text)
 {
-    printtext(NULL, NULL, MSGLEVEL_CLIENTERROR, ret_texts[GPOINTER_TO_INT(error)]);
+    printformat(NULL, NULL, MSGLEVEL_CLIENTERROR, IRCTXT_PERL_ERROR, text);
     return TRUE;
 }
 
@@ -592,7 +606,7 @@ void ui_events_init(void)
     signal_add("plugin error load", (SIGNAL_FUNC) event_plugin_load_error);
     signal_add("plugin error already loaded", (SIGNAL_FUNC) event_plugin_already_loaded);
     signal_add("plugin error not loaded", (SIGNAL_FUNC) event_plugin_not_loaded);
-    signal_add("error command", (SIGNAL_FUNC) event_cmderror);
+    signal_add("perl error", (SIGNAL_FUNC) event_perl_error);
 }
 
 void ui_events_deinit(void)
@@ -626,5 +640,5 @@ void ui_events_deinit(void)
     signal_remove("plugin error load", (SIGNAL_FUNC) event_plugin_load_error);
     signal_remove("plugin error already loaded", (SIGNAL_FUNC) event_plugin_already_loaded);
     signal_remove("plugin error not loaded", (SIGNAL_FUNC) event_plugin_not_loaded);
-    signal_remove("error command", (SIGNAL_FUNC) event_cmderror);
+    signal_remove("perl error", (SIGNAL_FUNC) event_perl_error);
 }
