@@ -118,6 +118,12 @@ gchar *strip_codes(gchar *input)
     {
 	if (*p == '\003')
 	{
+	    if (p[1] < 17 && p[1] > 0)
+	    {
+		p++;
+		if (p[1] < 17 && p[1] > 0) p++;
+		continue;
+	    }
 	    loop_state = IN_COLOR_CODE;
 	    continue;
 	}
@@ -199,6 +205,13 @@ static gboolean expand_styles(GString *out, gchar **format, SERVER_REC *server,
             g_string_truncate(out, 0);
 	    break;
 
+	case '|':
+	    /* Indent here mark */
+	    g_string_append_c(out, 4);
+            g_string_append_c(out, -1);
+            g_string_append_c(out, 4);
+	    break;
+
 	case 'F':
             /* flashing - ignore */
 	    break;
@@ -288,9 +301,9 @@ void output_format_text_args(GString *out, SERVER_REC *server, gchar *channel, g
                 break;
             case FORMAT_FLOAT:
                 {
-                    gfloat f = (gfloat) va_arg(args, gfloat);
-                    params[num] = g_new(gfloat, 1);
-                    memcpy(params[num], &f, sizeof(gfloat));
+                    gdouble f = (gdouble) va_arg(args, gdouble);
+                    params[num] = g_new(gdouble, 1);
+                    memcpy(params[num], &f, sizeof(gdouble));
                 }
                 break;
         }
@@ -318,7 +331,7 @@ void output_format_text_args(GString *out, SERVER_REC *server, gchar *channel, g
         {
             /* found a parameter! */
             gint num, argnum;
-	    gboolean cut, rightpad;
+	    gboolean cut, rightalign;
 	    gchar pad;
 
 	    if (!isdigit(*str) && *str != '[')
@@ -328,7 +341,7 @@ void output_format_text_args(GString *out, SERVER_REC *server, gchar *channel, g
 		continue;
 	    }
 
-	    rightpad = cut = FALSE; pad = ' ';
+	    rightalign = cut = FALSE; pad = ' ';
             if (*str != '[')
 		argnum = 0;
             else
@@ -340,7 +353,7 @@ void output_format_text_args(GString *out, SERVER_REC *server, gchar *channel, g
 		    if (*str == '!')
 			cut = FALSE;
 		    else if (*str == '-')
-                        rightpad = TRUE;
+                        rightalign = TRUE;
                     str++;
 		}
 		if (!isdigit(*str)) break;
@@ -387,8 +400,7 @@ void output_format_text_args(GString *out, SERVER_REC *server, gchar *channel, g
 		    arg = g_strdup_printf("%ld", *((glong *) params[num]));
                     break;
 		case FORMAT_FLOAT:
-		    /* FIXME: floats don't work for some reason */
-		    arg = g_strdup_printf("%0.2f", *((gfloat *) params[num]));
+		    arg = g_strdup_printf("%0.2f", *((gdouble *) params[num]));
 		    break;
 		default:
 		    arg = NULL;
@@ -404,9 +416,9 @@ void output_format_text_args(GString *out, SERVER_REC *server, gchar *channel, g
 		if (cut && argnum > 0 && len > argnum)
 		    arg[argnum] = '\0';
 
-		if (!rightpad)
+		if (rightalign)
 		{
-		    /* left padding, add pad characters */
+		    /* right alignment, add pad characters */
 		    while (argnum > len)
 		    {
 			g_string_append_c(out, pad);
@@ -417,9 +429,9 @@ void output_format_text_args(GString *out, SERVER_REC *server, gchar *channel, g
 		/* add the text */
 		g_string_append(out, arg);
 
-		if (rightpad)
+		if (!rightalign)
 		{
-		    /* right padding, add pad characters */
+		    /* left alignment, add pad characters */
 		    while (argnum > len)
 		    {
 			g_string_append_c(out, pad);
@@ -450,7 +462,7 @@ static void output_format_text(GString *out, SERVER_REC *server, gchar *channel,
     va_end(args);
 }
 
-static void add_timestamp(GString *out, SERVER_REC *server, gchar *channel, gint level)
+static void add_timestamp(WINDOW_REC *window, GString *out, SERVER_REC *server, gchar *channel, gint level)
 {
     time_t t;
     struct tm *tm;
@@ -460,12 +472,19 @@ static void add_timestamp(GString *out, SERVER_REC *server, gchar *channel, gint
 	  (toggle_show_timestamps || (toggle_show_msgs_timestamps && (level & MSGLEVEL_MSGS) != 0))))
 	return;
 
-    tmp = g_string_new(NULL);
     t = time(NULL);
-    tm = localtime(&t);
 
+    if ((t - window->last_timestamp) <= setup_get_int("timestamp_timeout"))
+    {
+	window->last_timestamp = t;
+	return;
+    }
+    window->last_timestamp = t;
+
+    tmp = g_string_new(NULL);
+    tm = localtime(&t);
     output_format_text(tmp, server, channel, level, IRCTXT_TIMESTAMP,
-                       tm->tm_year+1900, tm->tm_mon, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
+                       tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
 
     /* insert the timestamp just after \n */
     g_string_prepend(out, tmp->str);
@@ -526,12 +545,6 @@ void printtext(SERVER_REC *server, gchar *channel, gint level, gchar *str, ...)
                     g_string_sprintfa(out, "%0.2f", f);
                     break;
                 }
-            case 'c':
-                {
-                    gchar c = (gchar )va_arg(args, gint);
-                    g_string_append_c(out, c);
-                    break;
-                }
             case 'u':
                 {
                     guint d = (guint) va_arg(args, guint);
@@ -622,21 +635,40 @@ static gboolean sig_print_text(SERVER_REC *server, gchar *channel, gpointer leve
 
     newline(chanrec);
 
-    /* add timestamp */
     out = g_string_new(str);
-    add_timestamp(out, server, channel, GPOINTER_TO_INT(level));
+    if (server != NULL && servers != NULL && servers->next != NULL &&
+	((window->active->type != CHANNEL_TYPE_CHANNEL &&
+	  window->active->type != CHANNEL_TYPE_QUERY) ||
+	 window->active->server != server))
+    {
+	/* connected to more than one server and active server isn't the
+	   same where the message came or we're in status/msgs/empty window -
+	   prefix with a [server tag] */
+	gchar *str;
+
+	str = g_strdup_printf("[%s] ", server->tag);
+	g_string_prepend(out, str);
+	g_free(str);
+    }
+
+    add_timestamp(window, out, server, channel, GPOINTER_TO_INT(level));
+
     dup = str = out->str;
     g_string_free(out, FALSE);
 
     while (*str != '\0')
     {
-        for (ptr = str; *ptr != '\0'; ptr++)
-            if (*ptr == 2 || *ptr == 3 || *ptr == 4 || *ptr == 7 || *ptr == 15 || *ptr == 22 || *ptr == 27 || *ptr == 31)
+	for (ptr = str; *ptr != '\0'; ptr++)
+	{
+            if (*ptr == 2 || *ptr == 3 || *ptr == 4 || *ptr == 6 || *ptr == 7 || *ptr == 15 || *ptr == 22 || *ptr == 27 || *ptr == 31)
             {
                 type = *ptr;
                 *ptr++ = '\0';
                 break;
-            }
+	    }
+
+            *ptr = (gchar) translation_in[(gint) (guchar) *ptr];
+	}
 
         if (type == 7)
         {
@@ -649,7 +681,7 @@ static gboolean sig_print_text(SERVER_REC *server, gchar *channel, gpointer leve
             signal_emit("gui print text", 6, chanrec,
                         GINT_TO_POINTER(fgcolor), GINT_TO_POINTER(bgcolor),
                         GINT_TO_POINTER(flags), str, level);
-            flags &= ~PRINTFLAG_BEEP;
+            flags &= ~(PRINTFLAG_BEEP|PRINTFLAG_INDENT);
         }
         if (*ptr == '\0') break;
 
@@ -660,6 +692,11 @@ static gboolean sig_print_text(SERVER_REC *server, gchar *channel, gpointer leve
                 if (!toggle_hide_text_style)
                     flags ^= PRINTFLAG_BOLD;
                 break;
+	    case 6:
+		/* blink */
+                if (!toggle_hide_text_style)
+                    flags ^= PRINTFLAG_BLINK;
+                break;
 	    case 15:
                 /* remove all styling */
 		flags &= PRINTFLAG_BEEP;
@@ -680,13 +717,14 @@ static gboolean sig_print_text(SERVER_REC *server, gchar *channel, gpointer leve
                 break;
             case 4:
                 /* user specific colors */
+		flags &= ~PRINTFLAG_MIRC_COLOR;
                 if ((signed char) *ptr == -1)
                 {
 		    ptr++;
 		    if ((signed char) *ptr == -1)
 		    {
 			fgcolor = bgcolor = -1;
-			flags = 0;
+			flags &= PRINTFLAG_INDENT;
 		    }
                     else if (*ptr == 1)
                         flags ^= PRINTFLAG_BOLD;
@@ -694,6 +732,8 @@ static gboolean sig_print_text(SERVER_REC *server, gchar *channel, gpointer leve
                         flags ^= PRINTFLAG_UNDERLINE;
                     else if (*ptr == 3)
                         flags ^= PRINTFLAG_REVERSE;
+                    else if (*ptr == 4)
+                        flags |= PRINTFLAG_INDENT;
                 }
                 else
 		{
@@ -824,7 +864,7 @@ static gint sig_check_daychange(void)
 	WINDOW_REC *win = tmp->data;
 
 	printformat(win->active->server, win->active->name, MSGLEVEL_NEVER,
-		    IRCTXT_DAYCHANGE, tm->tm_mday, tm->tm_mon, 1900+tm->tm_year);
+		    IRCTXT_DAYCHANGE, tm->tm_mday, tm->tm_mon+1, 1900+tm->tm_year);
     }
     lastday = tm->tm_mday;
     return TRUE;
