@@ -8,6 +8,8 @@ typedef struct
     GList *channels;
     CHANNEL_REC *active;
 
+    GSList *waiting_channels; /* list of "<server tag> <channel>" */
+
     gint lines;
     gboolean destroying;
 
@@ -16,6 +18,7 @@ typedef struct
     gint histlines;
 
     gint new_data;
+    time_t last_timestamp; /* When was last timestamp printed */
 
     gpointer gui_data;
 }
@@ -23,7 +26,7 @@ WINDOW_REC;
 
 extern GList *windows;
 
-WINDOW_REC *ui_window_create(CHANNEL_REC *channel);
+WINDOW_REC *ui_window_create(CHANNEL_REC *channel, gboolean automatic);
 void ui_window_destroy(WINDOW_REC *window);
 
 void ui_windows_init(void);
